#ifndef _H_GLZ_ENCODER
#define _H_GLZ_ENCODER

/* Manging the lz encoding using a dictionary that is shared among encoders */

#include <stdint.h>
#include "lz_common.h" 
#include "glz_encoder_dictionary.h"
#include "glz_encoder_config.h"

typedef void GlzEncoderContext; 

GlzEncoderContext *glz_encoder_create(uint8_t id, GlzEncDictContext *dictionary, 
                                    GlzEncoderUsrContext *usr);

void glz_encoder_destroy(GlzEncoderContext *opaque_encoder);

/*
	assumes width is in pixels and stride is in bytes
    usr_context       : when an image is released from the window due to capicity overflow,
                        usr_context is given as a parmater to the free_image callback.
    o_enc_dict_context: if glz_enc_dictionary_remove_image is called, it should be
                        called with the o_enc_dict_context that is associated with 
                        the image.

	return: the number of bytes in the compressed data and sets o_enc_dict_context

	NOTE  :	currently supports only rgb images in which width*bytes_per_pixel = stride OR
			palette images in which stride eqauls the min number of bytes to hold a line.
			The stride should be > 0
*/
int glz_encode(GlzEncoderContext *opaque_encoder, LzImageType type, int width, int height, 
              int top_down, uint8_t *lines, unsigned int num_lines, int stride,
              uint8_t *io_ptr, unsigned int num_io_bytes, GlzUsrImageContext *usr_context, 
              GlzEncDictImageContext **o_enc_dict_context);
    

#endif // _H_GLZ_ENCODER
